from typing import Optional

from uuid import UUID
from pydantic import Field

from pydantic_api.base import BaseModel
from .base import NotionPaginatedData, StartCursor, PageSize
from ..objects import UserObject, BotUserObject


class ListAllUsersRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/get-users"""

    start_cursor: Optional[StartCursor] = None
    page_size: Optional[PageSize] = None


ListAllUsersResponse = NotionPaginatedData[UserObject]
"""Reference: https://developers.notion.com/reference/get-users"""


class RetrieveUserRequest(BaseModel):
    user_id: UUID = Field(..., description="Identifier for a Notion user")


RetrieveUserResponse = UserObject
"""Reference: https://developers.notion.com/reference/get-user"""


class RetrieveBotUserRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/get-self"""

    pass


RetrieveBotUserResponse = BotUserObject
"""Reference: https://developers.notion.com/reference/get-self"""


__all__ = [
    "ListAllUsersRequest",
    "ListAllUsersResponse",
    "RetrieveUserRequest",
    "RetrieveUserResponse",
    "RetrieveBotUserRequest",
    "RetrieveBotUserResponse",
]



"""
Reference: 

- https://developers.notion.com/reference/file-object
- https://developers.notion.com/reference/page-property-values#files
"""

from datetime import datetime
from typing import Literal, Optional, Annotated, Union

from pydantic import Field, HttpUrl
from pydantic_api.base import BaseModel


FileObjectTypeLiteral = Literal["file", "external"]
"""Reference: https://developers.notion.com/reference/file-object"""


class _FileExternal(BaseModel):
    """The `external` field of a `ExternalFileObject`."""

    url: HttpUrl


class _FileUploaded(BaseModel):
    """The `file` field of a `UploadedFileObject`."""

    url: HttpUrl
    expiry_time: Optional[datetime] = None


class _BaseFileObject(BaseModel):
    type: FileObjectTypeLiteral
    name: Optional[str] = Field(
        None, description="The name of the file to discriminate it."
    )
    """`name` field is not mentioned in https://developers.notion.com/reference/file-object, but mentioned in: https://developers.notion.com/reference/page-property-values#files"""


class ExternalFileObject(_BaseFileObject):
    """A file object corresponding to an external file that has been linked to in Notion"""

    type: Literal["external"] = "external"
    external: _FileExternal

    @classmethod
    def new(cls, url: HttpUrl):
        """
        Args:
            url (str): The URL of the external file.

        Returns:
            ExternalFileObject: An instance of ExternalFileObject.
        """
        return cls(external=_FileExternal(url=url))


class UploadedFileObject(_BaseFileObject):
    """A file object corresponding to a file that has been uploaded to Notion"""

    type: Literal["file"] = "file"
    file: _FileUploaded

    @classmethod
    def new(cls, url: HttpUrl, expire_time: Optional[datetime] = None):
        """
        Args:
            url (str): The URL of the uploaded file.
            expire_time (datetime, optional): The time at which the file will expire. Defaults to None.

        Returns:
            UploadedFileObject: An instance of UploadedFileObject
        """
        return cls(file=_FileUploaded(url=url, expiry_time=expire_time))


FileObject = Annotated[
    Union[ExternalFileObject, UploadedFileObject], Field(discriminator="type")
]
"""FileObject. Reference: https://developers.notion.com/reference/file-object"""


class FileObjectFactory:
    @classmethod
    def new_external(cls, url: HttpUrl) -> ExternalFileObject:
        """
        Create a new External File Object

        Args:
            url (str): The URL of the external file.

        Returns:
            ExternalFileObject: An instance of ExternalFileObject.
        """
        return ExternalFileObject.new(url=url)

    @classmethod
    def new_uploaded(
        cls, url: HttpUrl, expire_time: Optional[datetime] = None
    ) -> UploadedFileObject:
        """
        Create a new Uploaded File Object

        Args:
            url (str): The URL of the uploaded file.
            expire_time (datetime, optional): The time at which the file will expire. Defaults to None.

        Returns:
            UploadedFileObject: An instance of UploadedFileObject
        """
        return UploadedFileObject.new(url=url, expire_time=expire_time)


__all__ = [
    "FileObjectTypeLiteral",
    "ExternalFileObject",
    "UploadedFileObject",
    "FileObject",
    "FileObjectFactory",
    "_BaseFileObject",
    "_FileExternal",
    "_FileUploaded",
]



from typing import Union, Dict, List, Any, Optional

from uuid import UUID
from pydantic import BaseModel, Field

from .base import NotionPaginatedData, StartCursor, PageSize
from ..objects import (
    PageParentObject,
    DatabaseParentObject,
    IconObject,
    CoverObject,
    Page,
    PageProperty,
    CheckboxProperty,
    CreatedByProperty,
    CreatedTimeProperty,
    DateProperty,
    EmailProperty,
    FilesProperty,
    LastEditedByProperty,
    LastEditedTimeProperty,
    MultiSelectProperty,
    NumberProperty,
    PhoneNumberProperty,
    SelectProperty,
    StatusProperty,
    URLProperty,
    TitleProperty,
    RichTextProperty,
    PeopleProperty,
    RelationProperty,
    RollupProperty,
    UniqueIDProperty,
    FormulaProperty,
    VerificationProperty,
)


class CreatePageRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/post-page"""

    parent: Union[PageParentObject, DatabaseParentObject] = Field(
        ...,
        description="The parent page or database where the new page is inserted, represented as a JSON object with a page_id or database_id key, and the corresponding ID.",
    )
    properties: Dict[str, PageProperty] = Field(
        ...,
        description="The values of the page’s properties. If the parent is a database, then the schema must match the parent database's properties. If the parent is a page, then the only valid object key is title.",
    )
    # FIXME: implement block types
    children: List[Any] = Field(
        default_factory=list,
        description="The content to be rendered on the new page, represented as an array of block objects.",
    )
    icon: Optional[IconObject] = Field(
        None,
        description="The icon of the new page. Either an emoji object or an external file object.",
    )
    cover: Optional[CoverObject] = Field(
        None,
        description="The cover image of the new page, represented as a file object.",
    )


CreatePageResponse = Page
"""Reference: https://developers.notion.com/reference/post-page"""


class RetrievePageRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/retrieve-a-page"""

    page_id: UUID = Field(
        ...,
        description="Identifier for a Notion page, a uuidv4 string. Reference: https://developers.notion.com/reference/page#keys",
    )
    filter_properties: Optional[List[str]] = Field(
        None,
        description="A list of page property value IDs associated with the page. Use this param to limit the response to a specific page property value or values. To retrieve multiple properties, specify each page property ID. For example: ?filter_properties=iAk8&filter_properties=b7dh.",
        examples=["iAk8", "b7dh"],
    )


RetrievePageResponse = Page
"""Reference: https://developers.notion.com/reference/retrieve-a-page"""

SimplePageProperty = Union[
    CheckboxProperty,
    CreatedByProperty,
    CreatedTimeProperty,
    DateProperty,
    EmailProperty,
    FilesProperty,
    LastEditedByProperty,
    LastEditedTimeProperty,
    MultiSelectProperty,
    NumberProperty,
    PhoneNumberProperty,
    SelectProperty,
    StatusProperty,
    URLProperty,
    RollupProperty,
    UniqueIDProperty,
    FormulaProperty,
    VerificationProperty,
]
"""Reference: https://developers.notion.com/reference/retrieve-a-page-property#simple-properties"""

PaginatedPageProperty = Union[
    TitleProperty,
    RichTextProperty,
    PeopleProperty,
    RelationProperty,
]
"""Reference: https://developers.notion.com/reference/retrieve-a-page-property#paginated-properties"""


class RetrievePagePropertyItemRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/retrieve-a-page-property"""

    page_id: UUID = Field(
        ...,
        description="Identifier for a Notion page",
    )
    property_id: str = Field(
        ...,
        description="Identifier for a page property.",
    )
    start_cursor: Optional[StartCursor] = None
    page_size: Optional[PageSize] = None


RetrievePagePropertyItemResponse = Union[
    SimplePageProperty, NotionPaginatedData[PaginatedPageProperty]
]
"""Reference: https://developers.notion.com/reference/retrieve-a-page-property"""


class UpdatePagePropertiesRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/patch-page"""

    page_id: UUID = Field(
        ...,
        description="The identifier for the Notion page to be updated.",
    )
    properties: Optional[Dict[str, PageProperty]] = Field(
        None,
        description="The property values to update for the page. The keys are the names or IDs of the property and the values are property values. If a page property ID is not included, then it is not changed.",
    )
    archived: Optional[bool] = Field(
        None,
        description="If true, the page is archived. If false, the page is unarchived.",
    )
    icon: Optional[IconObject] = Field(
        None,
        description="A page icon for the page. Supported types are external file object or emoji object.",
    )
    cover: Optional[CoverObject] = Field(
        None,
        description="A cover image for the page. Supported types are external file object.",
    )


UpdatePagePropertiesResponse = Page
"""Reference: https://developers.notion.com/reference/patch-page"""


__all__ = [
    "CreatePageRequest",
    "CreatePageResponse",
    "RetrievePageRequest",
    "RetrievePageResponse",
    "RetrievePagePropertyItemRequest",
    "RetrievePagePropertyItemResponse",
    "UpdatePagePropertiesRequest",
    "UpdatePagePropertiesResponse",
]



"""

Module: `pydantic_api.notion.models.endpoints`

Contains all data classes defined in the Notion API documentation, the *ENDPOINTS* section.

I.e. all Request and Response data models for the Notion API endpoints.

- Authentication: `pydantic_api.notion.models.endpoints.authentication`  (Not implemented)
- Blocks: `pydantic_api.notion.models.endpoints.blocks`  (Not implemented)
- Pages: `pydantic_api.notion.models.endpoints.pages`
- Databases: `pydantic_api.notion.models.endpoints.databases`
- Users: `pydantic_api.notion.models.endpoints.users`
- Comments: `pydantic_api.notion.models.endpoints.comments`
- Search: `pydantic_api.notion.models.endpoints.search`

"""

from .base import (
    PageSize,
    StartCursor,
    SortObject,
    SortObjectFactory,
    NotionPaginatedData,
    NotionPaginatedDataTypeLiteral,
    CheckboxFilterObject,
    DateFilterObject,
    FilesFilterObject,
    FormulaFilterObject,
    MultiSelectFilterObject,
    NumberFilterObject,
    PeopleFilterObject,
    RelationFilterObject,
    RichTextFilterObject,
    RollupFilterObject,
    SelectFilterObject,
    StatusFilterObject,
    TimestampFilterObject,
    UniqueIDFilterObject,
    FilterObject,
)
from .pages import (
    CreatePageRequest,
    CreatePageResponse,
    RetrievePageRequest,
    RetrievePageResponse,
    RetrievePagePropertyItemRequest,
    RetrievePagePropertyItemResponse,
    UpdatePagePropertiesRequest,
    UpdatePagePropertiesResponse,
)
from .users import (
    ListAllUsersRequest,
    ListAllUsersResponse,
    RetrieveUserRequest,
    RetrieveUserResponse,
    RetrieveBotUserRequest,
    RetrieveBotUserResponse,
)
from .blocks import (
    AppendBlockChildrenRequest,
    AppendBlockChildrenResponse,
    RetrieveBlockRequest,
    RetrieveBlockResponse,
    RetrieveBlockChildrenRequest,
    RetrieveBlockChildrenResponse,
    # UpdateBlockRequest,
    UpdateBlockResponse,
    DeleteBlockRequest,
    DeleteBlockResponse,
)
from .databases import (
    CreateDatabaseRequest,
    CreateDatabaseResponse,
    QueryDatabaseRequest,
    QueryDatabaseResponse,
    RetrieveDatabaseRequest,
    RetrieveDatabaseResponse,
    UpdateDatabaseRequest,
    UpdateDatabaseResponse,
)
from .comments import (
    CreateCommentRequest,
    CreateCommentResponse,
    RetrieveCommentsRequest,
    RetrieveCommentsResponse,
)
from .search import (
    SearchByTitleRequest,
    SearchByTitleResponse,
    SearchByTitleFilterObject,
)


__all__ = [
    # base
    "PageSize",
    "StartCursor",
    "NotionPaginatedDataTypeLiteral",
    "NotionPaginatedData",
    "SortObject",
    "SortObjectFactory",
    "CheckboxFilterObject",
    "DateFilterObject",
    "FilesFilterObject",
    "FormulaFilterObject",
    "MultiSelectFilterObject",
    "NumberFilterObject",
    "PeopleFilterObject",
    "RelationFilterObject",
    "RichTextFilterObject",
    "RollupFilterObject",
    "SelectFilterObject",
    "StatusFilterObject",
    "TimestampFilterObject",
    "UniqueIDFilterObject",
    "FilterObject",
    # pages
    "CreatePageRequest",
    "CreatePageResponse",
    "RetrievePageRequest",
    "RetrievePageResponse",
    "RetrievePagePropertyItemRequest",
    "RetrievePagePropertyItemResponse",
    "UpdatePagePropertiesRequest",
    "UpdatePagePropertiesResponse",
    # users
    "ListAllUsersRequest",
    "ListAllUsersResponse",
    "RetrieveUserRequest",
    "RetrieveUserResponse",
    "RetrieveBotUserRequest",
    "RetrieveBotUserResponse",
    # blocks
    "AppendBlockChildrenRequest",
    "AppendBlockChildrenResponse",
    "RetrieveBlockRequest",
    "RetrieveBlockResponse",
    "RetrieveBlockChildrenRequest",
    "RetrieveBlockChildrenResponse",
    # "UpdateBlockRequest",
    "UpdateBlockResponse",
    "DeleteBlockRequest",
    "DeleteBlockResponse",
    # databases
    "CreateDatabaseRequest",
    "CreateDatabaseResponse",
    "QueryDatabaseRequest",
    "QueryDatabaseResponse",
    "RetrieveDatabaseRequest",
    "RetrieveDatabaseResponse",
    "UpdateDatabaseRequest",
    "UpdateDatabaseResponse",
    # comments
    "CreateCommentRequest",
    "CreateCommentResponse",
    "RetrieveCommentsRequest",
    "RetrieveCommentsResponse",
    # search
    "SearchByTitleFilterObject",
    "SearchByTitleRequest",
    "SearchByTitleResponse",
]



from typing import Annotated, Union

from pydantic import Field

from .text import TextRichTextObject
from .mention import MentionRichTextObject
from .equation import EquationRichTextObject

RichTextObject = Annotated[
    Union[TextRichTextObject, MentionRichTextObject, EquationRichTextObject],
    Field(discriminator="type"),
]


# Factory class for Devlopers
class RichTextObjectFactory:
    @classmethod
    def new_text(cls, content: str, link: dict = None):
        return TextRichTextObject.new(content=content, link=link)

    @classmethod
    def new_equation(cls, expression: str):
        return EquationRichTextObject.new(expression=expression)

    @classmethod
    def new_mention(cls):
        raise NotImplementedError(
            f"Factory method for Mention object is not implemented yet."
        )


__all__ = [
    "RichTextObject",
    "RichTextObjectFactory",
]



from typing import Literal, Optional

from pydantic import Field

from pydantic_api.base import BaseModel


TextAnnotationColorLiteral = Literal[
    "default",
    "blue",
    "blue_background",
    "brown",
    "brown_background",
    "default",
    "gray",
    "gray_background",
    "green",
    "green_background",
    "orange",
    "orange_background",
    "pink",
    "pink_background",
    "purple",
    "purple_background",
    "red",
    "red_background",
    "yellow",
    "yellow_background",
]


class TextAnnotations(BaseModel):
    """Text style annotations for rich text."""

    bold: bool
    italic: bool
    strikethrough: bool
    underline: bool
    code: bool
    color: TextAnnotationColorLiteral


RichTextTypeLiteral = Literal["text", "mention", "equation"]


class BaseRichTextObject(BaseModel):
    """Base model for Rich Text."""

    type: RichTextTypeLiteral
    annotations: Optional[TextAnnotations] = Field(
        None, description="Formatting style for the text"
    )
    plain_text: Optional[str] = Field(None)
    href: Optional[str] = Field(None, description="Hyperlink for the text")


__all__ = [
    "TextAnnotationColorLiteral",
    "TextAnnotations",
    "RichTextTypeLiteral",
    "BaseRichTextObject",
]



"""
Reference: https://developers.notion.com/reference/user
"""

from typing import Optional, Literal, Union, Annotated
from uuid import UUID
from pydantic import Field, EmailStr

from pydantic_api.base import BaseModel


UserObjectTypeLiteral = Literal["person", "bot"]


class PartialUser(BaseModel):
    """Partial user object.

    PartialUser is not mentioned in https://developers.notion.com/reference/user, but mentioned in the following pages(maybe include more):

    - https://developers.notion.com/reference/page
    - https://developers.notion.com/reference/database

    PartialUser is usually used as `created_by`, `last_edited_by`, `owner`, etc.
    """

    object: Literal["user"] = "user"
    id: UUID = Field(..., description="Unique identifier for this user.")


class _BaseUserObject(PartialUser):
    """Base class for all User objects."""

    type: Optional[UserObjectTypeLiteral] = Field(
        None, description='Type of the user. Possible values are "person" and "bot".'
    )
    name: Optional[str] = Field(
        None, description="User's name, as displayed in Notion."
    )
    avatar_url: Optional[str] = Field(None, description="Chosen avatar image.")


class _PersonData(BaseModel):
    """The `person` field of a `PersonUser` object."""

    email: Optional[EmailStr] = None
    """I set email to Optional because: https://developers.notion.com/reference/page-property-values#last_edited_by and https://developers.notion.com/reference/page-property-values#created_by"""


class PersonUserObject(_BaseUserObject):
    """A person user object."""

    type: Literal["person"] = "person"
    person: _PersonData = Field(default_factory=_PersonData)


BotOwnerTypeLiteral = Literal["workspace", "user"]


class _BotWorkspaceOwner(BaseModel):
    """The `owner` field for a workspace-owned bot."""

    type: Literal["workspace"] = "workspace"
    workspace: Literal[True] = True


class _BotUserOwner(BaseModel):
    """The `owner` field for a user-owned bot."""

    type: Literal["user"] = "user"


_BotOwner = Union[_BotWorkspaceOwner, _BotUserOwner]
"""The owner of the bot. Either a `workspace` or a `user`."""


class _BotData(BaseModel):
    """The `bot` field of a `BotUser` object."""

    # owner: _BotOwner
    owner: Optional[_BotOwner] = Field(
        None,
        description="The owner of the bot. Now I set it optional as a workaround, since I found sometimes `bot` might be an empty object in practice, which is not documented in the official API reference.",
    )
    workspace_name: Optional[str] = Field(
        None,
        description='If the owner.type is "workspace", then workspace.name identifies the name of the workspace that owns the bot. If the owner.type is "user", then workspace.name is null.',
    )


class BotUserObject(_BaseUserObject):
    """A bot user object."""

    type: Literal["bot"] = "bot"
    bot: _BotData = Field(
        default_factory=_BotData,
        description="Additional data about the bot, including owner information.",
    )


UserObject = Annotated[
    Union[PersonUserObject, BotUserObject], Field(discriminator="type")
]
"""
Union of all User objects: `PersonUser` and `BotUser`.
Reference: https://developers.notion.com/reference/user
"""

__all__ = [
    "UserObjectTypeLiteral",
    "PersonUserObject",
    "BotUserObject",
    "UserObject",
    "PartialUser",
]



"""
Reference: https://developers.notion.com/reference/page-property-values
"""

from typing import Literal, Union, List, Optional, Annotated
from datetime import datetime

from uuid import UUID
from pydantic import Field, EmailStr, HttpUrl

from pydantic_api.base import BaseModel
from ..user import UserObject
from ..file import FileObject
from ..block import RichTextObject, RichTextObjectFactory
from .common import (
    SelectOption,
    StatusOption,
    ColorLiteral,
    PagePropertyTypeLiteral,
    FormulaValueTypeLiteral,
    RollupTypeLiteral,
    RollupFunctionLiteral,
    VerificationStateLiteral,
)


class BasePageProperty(BaseModel):
    id: Optional[str] = Field(
        None,
        description="An underlying identifier for the property. id may be used in place of name when creating or updating pages. id remains constant when the property name changes.",
    )
    type: PagePropertyTypeLiteral


# checkbox, Refer to https://developers.notion.com/reference/page-property-values#checkbox
class ButtonProperty(BasePageProperty):
    type: Literal["button"] = "button"
    button: dict = Field(default_factory=dict)


class CheckboxProperty(BasePageProperty):
    type: Literal["checkbox"] = "checkbox"
    checkbox: bool = Field(
        ..., description="Whether the checkbox is checked (true) or unchecked (false)."
    )


# created_by, Refer to https://developers.notion.com/reference/page-property-values#created_by
class CreatedByProperty(BasePageProperty):
    type: Literal["created_by"] = "created_by"
    created_by: UserObject


# created_time, Refer to https://developers.notion.com/reference/page-property-values#created_time
class CreatedTimeProperty(BasePageProperty):
    type: Literal["created_time"] = "created_time"
    created_time: datetime


# date, Refer to https://developers.notion.com/reference/page-property-values#date
class DateValue(BaseModel):
    """Represents the value of a date property."""

    start: datetime = Field(..., description="The start of the date or date range.")
    end: Optional[datetime] = Field(
        None, description="The end of the date range. If None, the date is not a range."
    )
    time_zone: Optional[str] = Field(None, description="")


class DateProperty(BasePageProperty):
    """A page property of type 'date'."""

    type: Literal["date"] = "date"
    date: DateValue = Field(
        ...,
        description="The value of the date property, including start and optional end.",
    )

    @classmethod
    def new(
        cls,
        start: datetime,
        end: Optional[datetime] = None,
        time_zone: Optional[str] = None,
    ):
        return cls(date=DateValue(start=start, end=end, time_zone=time_zone))


# email: Refer to https://developers.notion.com/reference/page-property-values#email
class EmailProperty(BasePageProperty):
    type: Literal["email"] = "email"
    email: EmailStr


# files: Refer to https://developers.notion.com/reference/page-property-values#files
class FilesProperty(BasePageProperty):
    """Please note that when post or patch, FileObject must provide: `name` and `external` at the same time."""

    type: Literal["files"] = "files"
    files: List[FileObject]


# formula: Refer to https://developers.notion.com/reference/page-property-values#formula
class _BaseFormulaData(BaseModel):
    type: FormulaValueTypeLiteral


class BooleanFormulaData(_BaseFormulaData):
    type: Literal["boolean"] = "boolean"
    boolean: bool


class DateFormulaData(_BaseFormulaData):
    type: Literal["date"] = "date"
    date: DateValue


class NumberFormulaData(_BaseFormulaData):
    type: Literal["number"] = "number"
    number: float


class StringFormulaData(_BaseFormulaData):
    type: Literal["string"] = "string"
    string: str


FormulaData = Annotated[
    Union[BooleanFormulaData, DateFormulaData, NumberFormulaData, StringFormulaData],
    Field(discriminator="type"),
]


class FormulaProperty(BasePageProperty):
    type: Literal["formula"] = "formula"
    formula: FormulaData


# last_edited_by: Refer to https://developers.notion.com/reference/page-property-values#last_edited_by
class LastEditedByProperty(BasePageProperty):
    type: Literal["last_edited_by"] = "last_edited_by"
    last_edited_by: UserObject


# last_edited_time: Refer to https://developers.notion.com/reference/page-property-values#last_edited_time
class LastEditedTimeProperty(BasePageProperty):
    type: Literal["last_edited_time"] = "last_edited_time"
    last_edited_time: datetime


# multi_select: Refer to https://developers.notion.com/reference/page-property-values#multi_select
class MultiSelectProperty(BasePageProperty):
    type: Literal["multi_select"] = "multi_select"
    multi_select: List[SelectOption]


# number: Refer to https://developers.notion.com/reference/page-property-values#number
class NumberProperty(BasePageProperty):
    type: Literal["number"] = "number"
    number: float


# people: Refer to https://developers.notion.com/reference/page-property-values#people
class PeopleProperty(BasePageProperty):
    type: Literal["people"] = "people"
    people: List[UserObject]


# phone_number: Refer to https://developers.notion.com/reference/page-property-values#phone_number
class PhoneNumberProperty(BasePageProperty):
    type: Literal["phone_number"] = "phone_number"
    phone_number: Optional[str] = Field(
        None,
        description="A string representing a phone number. No phone number format is enforced.",
        examples=["415-867-5309"],
    )


# relation: Refer to https://developers.notion.com/reference/page-property-values#relation
class PageReference(BaseModel):
    id: UUID = Field(..., description="id of the referenced page")


class RelationProperty(BasePageProperty):
    type: Literal["relation"] = "relation"
    relation: List[PageReference]
    has_more: Optional[bool] = Field(
        None,
        description="If a relation has more than 25 references, then the has_more value for the relation in the response object is true. If a relation doesn’t exceed the limit, then has_more is false.",
    )

    @classmethod
    def create(cls, relation: List[UUID]):
        return cls(relation=[PageReference(id=uuid) for uuid in relation])


# rollup: Refer to https://developers.notion.com/reference/page-property-values#rollup
class BaseRollupData(BaseModel):
    type: RollupTypeLiteral
    function: RollupFunctionLiteral


class DateRollupData(BaseRollupData):
    type: Literal["date"] = "date"
    date: DateValue


class NumberRollupData(BaseRollupData):
    type: Literal["number"] = "number"
    number: float


class ArrayRollupData(BaseRollupData):
    # TODO: I don't know how array is represented
    type: Literal["array"] = "array"


class IncompleteRollupData(BaseRollupData):
    # TODO: I don't know how incomplete is represented
    type: Literal["incomplete"] = "incomplete"


class UnsupportedRollupData(BaseRollupData):
    # TODO: I don't know how unsupported is represented
    type: Literal["unsupported"] = "unsupported"


RollupData = Annotated[
    Union[
        DateRollupData,
        NumberRollupData,
        ArrayRollupData,
        IncompleteRollupData,
        UnsupportedRollupData,
    ],
    Field(discriminator="type"),
]


class RollupProperty(BasePageProperty):
    """
    Note:

    - The API does not support updating rollup page property values.
    - For rollup properties with more than 25 references, use the Retrieve a page property endpoint
    """

    type: Literal["rollup"] = "rollup"
    rollup: RollupData


# rich_text: Refer to https://developers.notion.com/reference/page-property-values#rich_text
class RichTextProperty(BasePageProperty):
    type: Literal["rich_text"] = "rich_text"
    rich_text: List[RichTextObject]


#  select: Refer to https://developers.notion.com/reference/page-property-values#select
class SelectProperty(BasePageProperty):
    type: Literal["select"] = "select"
    select: SelectOption

    @classmethod
    def create(
        cls,
        name: str,
        color: Optional[str] = None,
    ):
        return cls(select=SelectOption.create(name=name, color=color))


# status
class StatusProperty(BasePageProperty):
    type: Literal["status"] = "status"
    status: StatusOption

    @classmethod
    def create(
        cls,
        name: str,
        color: Optional[ColorLiteral] = "default",
    ):
        return cls(status=StatusOption.create(name=name, color=color))

    @classmethod
    def refer(
        cls,
        name: str,
    ):
        return cls(status=StatusOption.refer(name=name))


# title
class TitleProperty(BasePageProperty):
    type: Literal["title"] = "title"
    title: List[RichTextObject]

    @classmethod
    def new(
        cls,
        text: str,
    ):
        """
        Author Note: This method is a simple version now, does not support annotations and links, but it works for most cases.

        Args:
            text (str): The text content of the title.
        """
        return cls(title=[RichTextObjectFactory.new_text(content=text)])


# url
class URLProperty(BasePageProperty):
    type: Literal["url"] = "url"
    url: HttpUrl


# unique_id
class UniqueIDData(BaseModel):
    number: int = Field(..., description="The ID count (auto-incrementing).")
    prefix: Optional[str] = Field(
        None, description="An optional prefix to be applied to the unique ID."
    )


class UniqueIDProperty(BasePageProperty):
    """
    Note: Unique IDs can be read using the API with a GET page request, but they cannot be updated with the API, since they are auto-incrementing.
    """

    type: Literal["unique_id"] = "unique_id"
    unique_id: UniqueIDData


# verification
class VerificationData(BaseModel):
    state: VerificationStateLiteral
    verified_by: Optional[UserObject] = Field(
        None,
        description="If the page if verified, a User object will be included to indicate the user who verified the page.",
    )
    date: Optional[DateValue] = Field(
        None,
        description="If the page is verified, the date object will include the date the verification started (start). If an expiration date is set for the verification, an end date (end) will be included. (ISO 8601 date and time.)",
    )


class VerificationProperty(BasePageProperty):
    type: Literal["verification"] = "verification"
    verification: VerificationData


# Union types
PageProperty = Annotated[
    Union[
        ButtonProperty,
        CheckboxProperty,
        CreatedByProperty,
        CreatedTimeProperty,
        DateProperty,
        EmailProperty,
        FilesProperty,
        LastEditedByProperty,
        LastEditedTimeProperty,
        MultiSelectProperty,
        NumberProperty,
        PeopleProperty,
        PhoneNumberProperty,
        RichTextProperty,
        SelectProperty,
        StatusProperty,
        TitleProperty,
        URLProperty,
        FormulaProperty,
        RelationProperty,
        RollupProperty,
        UniqueIDProperty,
        VerificationProperty,
    ],
    Field(discriminator="type"),
]


__all__ = [
    "ButtonProperty",
    "CheckboxProperty",
    "CreatedByProperty",
    "CreatedTimeProperty",
    "DateProperty",
    "EmailProperty",
    "FilesProperty",
    "LastEditedByProperty",
    "LastEditedTimeProperty",
    "MultiSelectProperty",
    "NumberProperty",
    "PeopleProperty",
    "PhoneNumberProperty",
    "RichTextProperty",
    "SelectProperty",
    "StatusProperty",
    "TitleProperty",
    "URLProperty",
    "FormulaProperty",
    "RelationProperty",
    "RollupProperty",
    "UniqueIDProperty",
    "VerificationProperty",
    # Union types
    "PageProperty",
]



"""
Reference: https://developers.notion.com/reference/comment-object
"""

from typing import Literal, List, Union
from datetime import datetime

from uuid import UUID

from pydantic_api.base import BaseModel
from .user import PartialUser
from .block import RichTextObject
from .parent import BlockParentObject, PageParentObject


ParentOfComment = Union[BlockParentObject, PageParentObject]
"""Reference: https://developers.notion.com/reference/comment-object#all-comments"""


class CommentObject(BaseModel):
    """Comment Object

    Reference: https://developers.notion.com/reference/comment-object
    """

    object: Literal["comment"] = "comment"
    id: UUID
    parent: ParentOfComment
    discussion_id: UUID
    created_time: datetime
    last_edited_time: datetime
    created_by: PartialUser
    rich_text: List[RichTextObject]


__all__ = [
    "ParentOfComment",
    "CommentObject",
]



"""
Module: pydantic_api.notion.models.objects

Contains all data models defined in the Notion API documentation, the *OBJECTS* section.

- Block: `pydantic_api.notion.models.objects.block`
- Page: `pydantic_api.notion.models.objects.page`
- Database: `pydantic_api.notion.models.objects.database`
- Parent: `pydantic_api.notion.models.objects.parent`
- User: `pydantic_api.notion.models.objects.user`
- Comment: `pydantic_api.notion.models.objects.comment`
- Unfurl attribute(Link Previews): `pydantic_api.notion.models.objects.link_preview`
- File: `pydantic_api.notion.models.objects.file`
- Emoji: `pydantic_api.notion.models.objects.emoji`

"""

from .common import *
from .user import *
from .file import *
from .page import *
from .emoji import *
from .block import *
from .parent import *
from .comment import *
from .database import *
from .properties import *
from .link_preview import *
